/**
  * @file     plc_stat.c
  * @brief    PLCͳƹܵʵ.
  * @author   fengzhou
**/
/* includes -----------------------------------------------------------------*/
#include <string.h>
#include <rtthread.h>
#include "plc_debug.h"
#include "plc_stat.h"
#include "plc_timer.h"
#include "plc_comm.h"

/* Private define -----------------------------------------------------------*/
/* Private typedef ----------------------------------------------------------*/
/* Private macro ------------------------------------------------------------*/
/* Variables ----------------------------------------------------------------*/
static struct rt_semaphore	SemStat;                 /**<  ͳƹܵź */
unsigned char StatDataRegion[MAX_STAT_DATA_REGION_SIZE];  /**< ͳ */
unsigned int StatDataCnt = 0;      /**< ͳֽڼ  */
unsigned int StatBeginTime = 0;    /**< ͳƹʱ */
unsigned int StatLastEvtTime = 0;  /**< һ¼ʱ */
unsigned char StatFlag = 0;        /**< ͳƹܱ־ 0: ͳֹͣ1: ͳƽ. */
/* Private function declaration ---------------------------------------------*/
/* Functions ----------------------------------------------------------------*/
/**
  * @brief ͳƹܳʼ.
  */
void statInit(void)
{
	/* ͳƹź */
	rt_err_t err;
	err = rt_sem_init(&SemStat, 
	                  "", 
					  1, 
					  RT_IPC_FLAG_FIFO);
	RT_ASSERT(RT_EOK == err);
	memset((void*)StatDataRegion, 0, sizeof(StatDataRegion));
	StatDataCnt = 0;
	StatFlag = 0;
	StatBeginTime = 0;
	StatLastEvtTime = 0;
}

/**
  * @brief ͳƿʼڳʼͳ.
  */
void statBegin(void)
{
	rt_err_t err;

	err = rt_sem_take(&SemStat, RT_WAITING_FOREVER);
	RT_ASSERT(RT_EOK == err);
	
	//memset((void*)StatDataRegion, 0, sizeof(StatDataRegion));
	StatDataCnt = 0;
	debugOutput(PLC_EVT_STAT_START);
	StatBeginTime = plcTimeUsecGet();
	StatLastEvtTime = StatBeginTime;
	StatFlag = 1;

	rt_sem_release(&SemStat);
}

/**
  * @brief ͳƽ.
  */
void statStop(void)
{
	rt_err_t err;

	err = rt_sem_take(&SemStat, RT_WAITING_FOREVER);
	RT_ASSERT(RT_EOK == err);
	StatFlag = 0;
	rt_sem_release(&SemStat);
	debugOutput(PLC_EVT_STAT_STOP);
}

/**
  * @brief ͳֻ
  */
void statClear(void)
{
	rt_err_t err;
	if((StatFlag == 1) && (StatDataCnt > 0))
	{
		/* ȴź */
		err = rt_sem_take(&SemStat, RT_WAITING_FOREVER);
		RT_ASSERT(RT_EOK == err);
		StatDataCnt = 0;
		/* ͷź */
		rt_sem_release(&SemStat);
	}
}

/**
  * @brief  ͳ¼Ӻ.
  * @param  label ¼ǩ
  * @param  evt   ¼
  * @return ¼ʱ
  */
unsigned int statEventAdd(unsigned char label, STAT_EVT_TYPE_E evt)
{
	rt_err_t err;
	unsigned int evtTime;  /* ¼ʱ */
#if (STAT_DATA_SAVE_MODE == STAT_DATA_COMPACT_MODE) /* Խģʽͳ */
	unsigned int timeInc;  /* ʱ */
	unsigned char timeLen = 0;
#endif
	/* ȡ¼ʱ */
	evtTime = plcTimeUsecGet();

	if((StatFlag == 0) || (StatDataCnt >= (MAX_STAT_DATA_REGION_SIZE - SINGLE_STAT_DATA_SIZE)))
	{
		/* δͳƹܣ򲻹ռ洢µ¼ */
		return evtTime;
	}

	/* ȴź */
	err = rt_sem_take(&SemStat, RT_WAITING_FOREVER);
	RT_ASSERT(RT_EOK == err);

	/* дͳ */
	StatDataRegion[StatDataCnt++] = label;
	StatDataRegion[StatDataCnt] = evt;

#if (STAT_DATA_SAVE_MODE == STAT_DATA_COMPACT_MODE)	/* Խģʽͳ */	
	/* ʱ */
	timeInc = timeConsumedCalc(StatLastEvtTime, evtTime);
	/*ϴ¼ʱ*/
	StatLastEvtTime = evtTime;
	/* ʱ򳤶 */
	if(timeInc < 0x100){
		timeLen = 0;
	}else if(timeInc < 0x10000){
		timeLen = 1;
	}else if(timeInc < 0x1000000){
		timeLen = 2;
	}else{
		timeLen = 3;
	}
	StatDataRegion[StatDataCnt++] |= (timeLen << 6);
	do{
		StatDataRegion[StatDataCnt++] = (unsigned char)(timeInc & 0xFF);
		timeInc >>= 8;
	}while(timeLen--);
#else	/* ͨģʽͳ */
	StatDataCnt++;
	StatDataRegion[StatDataCnt++] = ((unsigned char)evtTime) & 0xFF;
	StatDataRegion[StatDataCnt++] = ((unsigned char)(evtTime >> 8)) & 0xFF;
	StatDataRegion[StatDataCnt++] = ((unsigned char)(evtTime >> 16)) & 0xFF;
	StatDataRegion[StatDataCnt++] = ((unsigned char)(evtTime >> 24)) & 0xFF;
#endif
	
	/* ͷź */
	rt_sem_release(&SemStat);

	return evtTime;
}

/**
  * @brief  ͳӺͳݴ洢ռ䲻ضϲݡ
  * @param data ָ
  * @param len  ݳ
  * @return ɹдֽ
  */
unsigned int statDataAdd(unsigned char* data, unsigned int len)
{
	unsigned int num = 0;
	unsigned int i;

	rt_err_t err;
	if((StatFlag == 0) || (StatDataCnt >= MAX_STAT_DATA_REGION_SIZE))
	{
		/* δͳƹܣ򲻹ռ洢µ¼ */
		return num;
	}

	/* ȴź */
	err = rt_sem_take(&SemStat, RT_WAITING_FOREVER);
	RT_ASSERT(RT_EOK == err);
	
	for(i = 0; i < len; i++){
		if(StatDataCnt < MAX_STAT_DATA_REGION_SIZE){
			StatDataRegion[StatDataCnt++] = data[i];
			num++;
		}else{
			break;
		}
	}
	/* ͷź */
	rt_sem_release(&SemStat);
	return num;
}


/**
  * @brief ͳݷ͸λ
  */
void statOutput(void)
{
	if((StatFlag == 1) && (StatDataCnt > 0))
	{
	    plcCommDataFrameSend(CMD_STAT_OUTPUT, StatDataRegion, StatDataCnt);
	}
}

